import { Injectable } from '@angular/core';
import {
  HttpRequest,
  HttpHandler,
  HttpEvent,
  HttpInterceptor,
  HttpErrorResponse
} from '@angular/common/http';
import { BehaviorSubject, Observable, of, throwError } from 'rxjs';
import { catchError, filter, switchMap, take, tap } from 'rxjs/operators';
import { AuthService } from './auth.service';

@Injectable()
export class HttpRequestInterceptor {
  private isRefreshing = false;
  private refreshTokenSubject: BehaviorSubject<string | null> = new BehaviorSubject<string | null>(null);
  private responseCache = new Map<string, any>();  // ใช้เก็บ cache ของ response

  constructor(private authService: AuthService) { }

  // intercept(req: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
  //   if (req.url.startsWith("./") || !sessionStorage.getItem("accessToken")) {
  //     return next.handle(req);
  //   } else {
  //     const authHeader = 'Bearer ' + sessionStorage.getItem("accessToken")
  //     const overideReq = {
  //       headers: req.headers.set('Authorization', authHeader),
  //       url: req.url,
  //     };
  //     const authReq = req.clone(overideReq);
  //     return next.handle(authReq).pipe(tap(response => {
  //       this.responseCache.set(req.urlWithParams, response)
  //     }));
  //   }

  // }

  intercept(req: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
    if (req.url.includes('/api/auth/refresh-token') || req.url.startsWith('./') || !sessionStorage.getItem('accessToken')) {
      return next.handle(req);
    }
    const authHeader = 'Bearer ' + sessionStorage.getItem('accessToken');
    const clonedReq = req.clone({
      headers: req.headers.set('Authorization', authHeader)
    });

    return next.handle(clonedReq).pipe(
      tap(response => {
        this.responseCache.set(req.urlWithParams, response); // เก็บ response cache
      }),
      catchError((error: HttpErrorResponse) => {
        if (error.status === 401) {
          return this.handle401Error(req, next);
        }
        return throwError(() => error);
      })
    );
  }

  private handle401Error(req: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
    if (this.isRefreshing) {
      return this.refreshTokenSubject.pipe(
        filter(token => token !== null),
        take(1),
        switchMap(token => {
          const clonedReq = req.clone({
            headers: req.headers.set('Authorization', `Bearer ${token}`)
          });
          return next.handle(clonedReq);
        })
      );
    } else {
      this.isRefreshing = true;
      this.refreshTokenSubject.next(null);

      return this.authService.refreshToken().pipe(
        switchMap(newTokens => {
          this.isRefreshing = false;
          this.refreshTokenSubject.next(newTokens.accessToken);

          const clonedReq = req.clone({
            headers: req.headers.set('Authorization', `Bearer ${newTokens.accessToken}`)
          });
          return next.handle(clonedReq);
        }),
        catchError(err => {
          this.isRefreshing = false;
          this.authService.logout();
          return throwError(() => new Error('Session expired, please log in again'));
        })
      );
    }
  }

}




